@@ -114,6 +114,13 @@ typedef size_t (*pony_custom_deserialise_fn)(void* p, void *addr);
 typedef void (*pony_dispatch_fn)(pony_ctx_t* ctx, pony_actor_t* actor,
   pony_msg_t* m);
 
+/** Behavior name function.
+ *
+ * Each actor has a behavior name function that translates a message id for an
+ * actor to a string behavior name that will run for that message.
+ */
+typedef char* (*pony_behavior_name_fn)(uint32_t id);
+
 /** Finaliser.
  *
  * An actor or object can supply a finaliser, which is called before it is
@@ -134,9 +141,14 @@ typedef const struct _pony_type_t
 {
   uint32_t id;
   uint32_t size;
+  size_t serialise_id;
   uint32_t field_count;
   uint32_t field_offset;
   void* instance;
+  #if defined(USE_RUNTIME_TRACING)
+  char* name;
+  pony_behavior_name_fn get_behavior_name;
+  #endif
   pony_trace_fn trace;
   pony_trace_fn serialise_trace;
   pony_serialise_fn serialise;
@@ -146,11 +158,18 @@ typedef const struct _pony_type_t
   pony_dispatch_fn dispatch;
   pony_final_fn final;
   uint32_t event_notify;
+  bool might_reference_actor;
   uintptr_t** traits;
   void* fields;
   void* vtable;
 } pony_type_t;
 
+/** Desc table lookup function.
+ *
+ * A function to convert `serialise_id`s to offsets in the desc table
+ */
+typedef uint32_t (*desc_offset_lookup_fn)(size_t serialise_id);
+
 /** Language feature initialiser.
  *
  * Contains initialisers for the various language features initialised by
@@ -177,35 +196,11 @@ typedef struct pony_language_features_init_t
 
   /// The total size of the descriptor_table array.
   size_t descriptor_table_size;
+
+  /// The function to translate `serialise_id`s to offsets in the desc_table
+  desc_offset_lookup_fn desc_table_offset_lookup;
 } pony_language_features_init_t;
 
-/** Padding for actor types.
- *
- * 56 bytes: initial header, not including the type descriptor
- * 52/104 bytes: heap
- * 48/88 bytes: gc
- * 28/0 bytes: padding to 64 bytes, ignored
- */
-#if INTPTR_MAX == INT64_MAX
-#ifdef USE_MEMTRACK
-#  define PONY_ACTOR_PAD_SIZE 280
-#else
-#  define PONY_ACTOR_PAD_SIZE 248
-#endif
-#elif INTPTR_MAX == INT32_MAX
-#ifdef USE_MEMTRACK
-#  define PONY_ACTOR_PAD_SIZE 176
-#else
-#  define PONY_ACTOR_PAD_SIZE 160
-#endif
-#endif
-
-typedef struct pony_actor_pad_t
-{
-  pony_type_t* type;
-  char pad[PONY_ACTOR_PAD_SIZE];
-} pony_actor_pad_t;
-
 /// The currently executing context.
 PONY_API pony_ctx_t* pony_ctx();
 
@@ -300,7 +295,7 @@ PONY_API ATTRIBUTE_MALLOC void* pony_alloc_large(pony_ctx_t* ctx, size_t size);
  * space, otherwise it allocates and copies. The old memory must have been
  * allocated via pony_alloc(), pony_alloc_small(), or pony_alloc_large().
  */
-PONY_API ATTRIBUTE_MALLOC void* pony_realloc(pony_ctx_t* ctx, void* p, size_t size);
+PONY_API ATTRIBUTE_MALLOC void* pony_realloc(pony_ctx_t* ctx, void* p, size_t size, size_t copy);
 
 /** Allocate memory with a finaliser.
  *
@@ -434,7 +429,7 @@ PONY_API void pony_traceunknown(pony_ctx_t* ctx, void* p, int m);
  *
  * Call this first. It will strip out command line arguments that you should
  * ignore and will return the remaining argc. Then create an actor and send it
- * a message, so that some initial work exists. Use pony_become() if you need
+ * a message, so that some initial work exists. Use ponyint_become() if you need
  * to send messages that require allocation and tracing.
  *
  * Then call pony_start().
@@ -483,7 +478,7 @@ PONY_API void pony_register_thread();
  */
 PONY_API void pony_unregister_thread();
 
-PONY_API int32_t pony_scheduler_index(pony_ctx_t* ctx);
+PONY_API int32_t pony_scheduler_index();
 
 /** Signals that the pony runtime may terminate.
  *
@@ -506,38 +501,12 @@ PONY_API void pony_exitcode(int code);
  */
 PONY_API int pony_get_exitcode();
 
-/**
- * If an actor is currently unscheduled, this will reschedule it. This is not
- * concurrency safe: only a single actor should reschedule another actor, and
- * it should be sure the target actor is actually unscheduled.
- */
-PONY_API void pony_schedule(pony_ctx_t* ctx, pony_actor_t* actor);
-
-/**
- * The actor will no longer be scheduled. It will not handle messages on its
- * queue until it is rescheduled, or polled on a context. This is not
- * concurrency safe: this should be done on the current actor or an actor that
- * has never been sent a message.
- */
-PONY_API void pony_unschedule(pony_ctx_t* ctx, pony_actor_t* actor);
-
-/**
- * Call this to "become" an actor on a non-scheduler context, i.e. from outside
- * the pony runtime. Following this, pony API calls can be made as if the actor
- * in question were the current actor, eg. pony_alloc, pony_send, pony_create,
- * etc. This should only be called with an unscheduled actor.
- *
- * This can be called with NULL to make no actor the "current" actor for a
- * thread.
- */
-PONY_API void pony_become(pony_ctx_t* ctx, pony_actor_t* actor);
-
 /**
  * Call this to handle an application message on an actor. This will do two
  * things: first, it will possibly gc, and second it will possibly handle a
  * pending application message.
  *
- * A thread must pony_become an actor before it can pony_poll.
+ * A thread must ponyint_become an actor before it can pony_poll.
  */
 PONY_API void pony_poll(pony_ctx_t* ctx);
 
